@@ -5,17 +5,12 @@ __author__ = 'Jens-Kristian Krogager'
 import numpy as np
 import matplotlib.pyplot as plt
 import copy
-import warnings
-import re
 
-from astropy.io import fits
 from lmfit import Parameters, Minimizer
 
 from . import Asplund
 from .components import Component
-from .fits_input import load_fits_spectrum, FormatError, MultipleSpectraWarning
 from . import hdf5_save
-from .limits import match_ion_state, match_ion_state_all, tau_percentile, tau_noise_range, equivalent_width
 from . import line_complexes
 from .line_complexes import fine_structure_complexes
 from .lines import Line, lineList
@@ -25,29 +20,9 @@ from .regions import Region, load_lsf
 from . import terminal_attributes as term
 from .voigt import evaluate_profile, evaluate_continuum
 
-from collections import namedtuple
-
-EquivalentWidth = namedtuple('EquivalentWidth', ['W_rest', 'W_err', 'logN', 'logN_err', 'logN_limit', 'line', 'sigma'])
 
 myfloat = np.float64
 
-def air2vac(air):
-    """
-    Air to vacuum conversion from Bengt Edlén 1953,
-    Journal of the Optical Society of America, Vol. 43, Issue 5, pp. 339-344.
-    """
-    if np.min(air) < 2000.:
-        raise ValueError("Input wavelength below valid range!")
-    air = np.array(air)
-    ij = (np.array(air) >= 2000)
-    out = np.array(air).copy()
-    s2 = (1.e4/air)**2
-    fact = 1.0 + 6.4328e-5 + 2.94981e-2/(146.0 - s2) + 2.5540e-4/(41.0 - s2)
-    # Alternative solution from VALD:
-    # fact = 1 + 8.336624e-5 + 0.02408927 / (130.106592 - s2) + 1.599740895e-4 / (38.925688 - s2),
-    out[ij] = air[ij]*fact[ij]
-    return out
-
 
 def mask_vel(dataset, line_tag, v1, v2):
     regions_of_line = dataset.find_line(line_tag)
@@ -156,9 +131,6 @@ class DataSet(object):
             The parameters will be defined during the call to :meth:`DataSet.prepare_dataset
             <VoigtFit.DataSet.prepare_dataset>` based on the defined components.
 
-        static_variables : `lmfit.Parameters`_
-            Parameter dictionary that holds fit variables other than those related to
-            components of absorption lines and continuum parameters.
 
 
         .. _lmfit.Parameters: https://lmfit.github.io/lmfit-py/parameters.html
@@ -171,7 +143,6 @@ class DataSet(object):
         # container for data chunks to be fitted
         # data should be added by calling method 'add_data'
         self.data = list()
-        self.data_filenames = list()
 
         self.verbose = True
 
@@ -203,7 +174,6 @@ class DataSet(object):
         self.best_fit = None
         self.minimizer = None
         self.pars = None
-        self.static_variables = Parameters()
         self.name = name
 
     def set_name(self, name):
@@ -214,7 +184,7 @@ class DataSet(object):
         """Returns the name of the DataSet."""
         return self.name
 
-    def add_spectrum(self, filename, res, airORvac='vac', normalized=False, mask_array=None, mask=None, use_mask=True, continuum=None, nsub=1, verbose=False, ext=None):
+    def add_spectrum(self, filename, res, normalized=False, mask=None, continuum=None, nsub=1):
         """
         Add spectral data from ASCII file (three or four columns accepted).
         This is a wrapper of the method `add_data`.
@@ -233,110 +203,38 @@ class DataSet(object):
             line-spread function for the given spectrum.
             See details in the documentation.
 
-        airORvac : string  {'vac' or 'air'}
-            Defines whether the input wavelengths are in 'air' or 'vacuum' units. If given as 'air'
-            the wavelengths will be converted to 'vacuum'.
-
         normalized : bool   [default = False]
             If the input spectrum is normalized this should be given as True
             in order to skip normalization steps.
 
-        mask_array : array, shape (n)
+        mask : array, shape (n)
             Boolean/int array defining the fitting regions.
             Only pixels with mask=True/1 will be included in the fit.
 
-        mask : Deprecated, use `mask_array`
-
-        use_mask : bool   [default = True]
-            If False, do not pass the mask array to the dataset. This may be useful if the mask defined in the spectrum
-            is not appropriate for the fitting purposes.
-
         continuum : array, shape (n)
             Continuum spectrum. The input spectrum will be normalized using
             this continuum spectrum.
 
-        nsub : integer   [default = 1]
+        nsub : integer
             Kernel subsampling factor relative to the data.
             This is only used if the resolution is given as a LSF file.
-
-        verbose : bool   [default = False]
-            Print status messages.
-
-        ext : int or string
-            Index or name of the extension in the HDU List. Only used if the input is a FITS file.
         """
-        if mask is not None:
-            warnings.warn("The keyword mask is deprecated. Use 'mask_array' instead", DeprecationWarning)
-            mask_array = mask
 
-        file_type = filename.split('.')[-1]
-        if file_type.lower() in ['fits', 'fit']:
-            with warnings.catch_warnings(record=True) as warning_list:
-                try:
-                    wl, spec, err, mask, _ = load_fits_spectrum(filename, ext=ext)
-                    has_multiSpecWarning = any([w.category is MultipleSpectraWarning for w in warning_list])
-                    if has_multiSpecWarning:
-                        # Show warning that there are multiple spectra
-                        print(term.red)
-                        print("\n [WARNING] - Several data tables were detected. The first extension was used.")
-                        print("")
-                        fits.info(filename)
-                        print(term.reset)
-                        print("")
+        spectrum = np.loadtxt(filename)
+        wl = spectrum[:, 0]
+        flux = spectrum[:, 1]
+        err = spectrum[:, 2]
+        if spectrum.shape[1] == 4 and mask is None:
+            mask = spectrum[:, 3]
 
-                except FormatError as error_msg:
-                    print(error_msg)
-                    print("")
-                    fits.info(filename)
-                    print("")
-                    fits_data = fits.getdata(filename)
-                    if isinstance(fits_data, fits.FITS_rec):
-                        # Show the Table Column definitions:
-                        print(fits_data.columns)
-                        raise
-
-        else:
-            data = np.loadtxt(filename)
-            if data.shape[1] == 3:
-                wl, spec, err = data.T
-                mask = np.ones_like(wl, dtype=bool)
-            elif data.shape[1] == 4:
-                wl, spec, err, mask = data.T
-                mask = mask.astype(bool)
-            elif data.shape[1] > 4:
-                wl = data[:, 0]
-                spec = data[:, 1]
-                err = data[:, 2]
-                mask = data[:, 3]
-                mask = mask.astype(bool)
-            else:
-                print(" [ERROR] - Not enough columns to load wavelength, flux and error")
-                print("           Check file format, must be three or more columns separated by blank space")
-                print("")
-                raise FormatError
-
-        if airORvac == 'air':
-            if verbose:
-                print(" Converting wavelength from air to vacuum.")
-            wl = air2vac(wl)
-
-        if continuum is not None and len(continuum) == len(spec):
-            spec = spec/continuum
+        if continuum is not None:
+            flux = flux/continuum
             err = err/continuum
             normalized = True
 
-        if mask_array is not None:
-            if len(mask_array) == len(spec):
-                mask = mask_array
-            else:
-                print("Wrong dimensions of the Input Mask: got %i pixels, spectrum has %i pixels" % (len(mask_array), len(spec)))
+        self.add_data(wl, flux, res, err=err, normalized=normalized, mask=mask, nsub=nsub)
 
-        if use_mask:
-            self.add_data(wl, spec, res, err=err, normalized=normalized, mask=mask, nsub=nsub, filename=filename)
-        else:
-            self.add_data(wl, spec, res, err=err, normalized=normalized, nsub=nsub, filename=filename)
-
-    def add_data(self, wl, flux, res, err=None, normalized=False, mask=None, nsub=1, filename=''):
+    def add_data(self, wl, flux, res, err=None, normalized=False, mask=None, nsub=1):
         """
         Add spectral data to the DataSet. This will be used to define fitting regions.
 
@@ -370,13 +268,7 @@ class DataSet(object):
         nsub : integer
             Kernel subsampling factor relative to the data.
             This is only used if the resolution is given as a LSF file.
-
-        filename : string
-            The filename from which the data originated. Optional but highly recommended.
-            Alternatively, use the method :meth:`DataSet.add_spectrum
-            <VoigtFit.DataSet.add_spectrum>`.
         """
-
         mask_warning = """
         All pixels in the spectrum have been masked out.
         Pixels to *include* in the fit should have value = 1.
@@ -408,7 +300,6 @@ class DataSet(object):
                           'error': err, 'res': res,
                           'norm': normalized, 'specID': specid,
                           'mask': mask, 'nsub': nsub})
-        self.data_filenames.append(filename)
 
     def reset_region(self, reg):
         """Reset the data in a given :class:`regions.Region` to use the raw input data."""
@@ -423,14 +314,13 @@ class DataSet(object):
         reg.wl = raw_data['wl'][cutout]
         reg.normalized = raw_data['norm']
 
-    def reset_all_regions(self, active_only=True):
+    def reset_all_regions(self):
         """
         Reset the data in all :class:`Regions <regions.Region>`
         defined in the DataSet to use the raw input data.
         """
         for reg in self.regions:
-            if reg.has_active_lines() or not active_only:
-                self.reset_region(reg)
+            self.reset_region(reg)
 
     def get_resolution(self, line_tag, verbose=False):
         """Return the spectral resolution for the fitting :class:`Region <regions.Region>`
@@ -690,7 +580,7 @@ class DataSet(object):
         line_tag : str
             Line tag of the transition that should be removed.
         """
-        if line_tag in self.all_lines and line_tag in [self.lines.keys()]:
+        if line_tag in self.all_lines and line_tag in self.lines.keys():
             self.all_lines.remove(line_tag)
             self.lines.pop(line_tag)
         else:
@@ -740,11 +630,11 @@ class DataSet(object):
                       line_tag)
 
     def remove_all_lines(self):
-        lines_to_remove = self.all_lines.copy()
+        lines_to_remove = self.lines.keys()
         for line_tag in lines_to_remove:
             self.remove_line(line_tag)
 
-    def normalize_line(self, line_tag, norm_method='spline', velocity=True):
+    def normalize_line(self, line_tag, norm_method='spline', velocity=False):
         """
         Normalize or re-normalize a given line
 
@@ -770,13 +660,7 @@ class DataSet(object):
 
         regions_of_line = self.find_line(line_tag)
         for region in regions_of_line:
-            if not region.normalized:
-                go_on = 0
-                while go_on == 0:
-                    go_on = region.normalize(norm_method=norm_method,
-                                             z_sys=z_sys)
-                    # region.normalize returns 1 when continuum is fitted
-            # region.normalize(norm_method=norm_method, z_sys=z_sys)
+            region.normalize(norm_method=norm_method, z_sys=z_sys)
 
     def mask_line(self, line_tag, reset=True, mask=None, telluric=True, velocity=False):
         """
@@ -822,7 +706,7 @@ class DataSet(object):
                 region.mask = mask
                 region.new_mask = False
             else:
-                if region.new_mask:
+                if region.new_mask and region.has_active_lines():
                     region.define_mask(z=self.redshift, dataset=self,
                                        telluric=telluric, z_sys=z_sys)
 
@@ -1005,9 +889,6 @@ class DataSet(object):
 
         return lines_for_ion
 
-    def reset_static_variables(self):
-        self.static_variables = Parameters()
-
     def reset_components(self, ion=None):
         """
         Reset component structure for a given ion.
@@ -1016,7 +897,7 @@ class DataSet(object):
         ----------
         ion : str   [default = None]
             The ion for which to reset the components: e.g., 'FeII', 'HI', 'CIa', etc.
-            If `None` is given, *all* components for *all* ions will be reset.
+            If `None`is given, *all* components for *all* ions will be reset.
         """
 
         if ion:
@@ -1117,9 +998,6 @@ class DataSet(object):
         is assumed to be unresolved, i.e., taken from the resolution.
         """
         regions_of_line = self.find_line(line_tag)
-        if len(regions_of_line) > 1:
-            print("\n Note -- The given line (%s) is defined in more than one spectral region")
-            print("        All regions will be shown one by one for interactive definition...\n")
         for region in regions_of_line:
             wl, flux, err, mask = region.unpack()
             plt.close('all')
@@ -1149,7 +1027,7 @@ class DataSet(object):
                 l0, f, gam = line.get_properties()
                 ion = line.ion
                 for comp in self.components[ion]:
-                    z = comp.z
+                    z = comp[0]
                     if velocity:
                         ax.axvline((l0*(z+1) - l_ref)/l_ref * 299792.458, ls=':', color='r', lw=0.4)
                     else:
@@ -1158,64 +1036,41 @@ class DataSet(object):
             if region.normalized:
                 c_level = 1.
             else:
-                ax.set_title("Click to Mark Average Continuum Level...\n(1 click)")
+                ax.set_title("Click to Mark Continuum Level...")
                 cont_level_point = plt.ginput(1, 30.)
                 c_level = cont_level_point[0][1]
                 ax.axhline(c_level, color='0.3', ls=':')
 
-            ax.set_title("Mark peak absorption for components of %s\nFinish with 'enter'" % line.tag)
+            ax.set_title("Mark central components for %s, finish with [enter]" % line.element)
             ax.set_xlabel(x_label)
             if region.normalized:
                 ax.set_ylabel(u"Normalized Flux")
             else:
                 ax.set_ylabel(u"Flux")
-            plt.draw()
-            plt.tight_layout()
-
+            comps = plt.ginput(-1, 60)
+            num = 0
             # Assume that components are unresolved:
             b = region.kernel_fwhm / 2.35482
-            comp_lines = list()
             comp_list = list()
-            success = False
-            while success is False:
-                comps = plt.ginput(-1, 60)
-                for x0, y0 in comps:
-                    if velocity:
-                        z0 = self.redshift + x0/299792.458 * (self.redshift + 1.)
-                    else:
-                        z0 = x0/line.l0 - 1.
-                    # Calculate logN from peak depth:
-                    y0 = max(y0/c_level, 0.01)
-                    logN = np.log10(-b * np.log(y0) / (1.4983e-15 * line.l0 * line.f))
-
-                    vline = ax.axvline(x0, color='darkblue', alpha=0.8)
-                    comp_lines.append(vline)
-                    comp_list.append([z0, b, logN])
-                plt.draw()
-
-                print("Are the shown components correct?")
-                ax.set_title("Are the shown components correct?")
-                plt.draw()
-                answer = input("[YES/no]  ")
-                if answer.lower() in ['', 'yes', 'y']:
-                    success = True
+            for x0, y0 in comps:
+                if velocity:
+                    z0 = self.redshift + x0/299792.458 * (self.redshift + 1.)
                 else:
-                    comp_list = list()
-                    for vline in comp_lines:
-                        vline.remove()
-                    comp_lines = list()
-                    ax.set_title("Mark peak absorption for components of %s\nFinish with 'enter'" % line.tag)
-                    plt.draw()
+                    z0 = x0/line.l0 - 1.
+                # Calculate logN from peak depth:
+                y0 = max(y0/c_level, 0.01)
+                logN = np.log10(-b * np.log(y0) / (1.4983e-15 * line.l0 * line.f))
+                print("Component %i:  z = %.6f   log(N) = %.2f" % (num, z0, logN))
+                ax.axvline(x0, color='darkblue', alpha=0.8)
+                comp_list.append([z0, b, logN])
+                num += 1
+            plt.draw()
 
             if len(comp_list) > 0:
-                print("Defining following components:")
-                print("(the lines below can be copied directly to the input file)\n")
                 for z, b, logN in comp_list:
-                    print("component %s  z=%.6f  b=%.1f  logN=%.2f" % (line.ion, z, b, logN))
-                    self.add_component(line.ion, z, b, logN)
+                    self.add_component(line.element, z, b, logN)
             else:
-                print("No components were defined. Are you sure you want to continue?")
-
+                pass
 
     def delete_component(self, ion, index):
         """Remove component of the given `ion` with the given `index`."""
@@ -1278,12 +1133,11 @@ class DataSet(object):
 
         for num, comp in enumerate(reference):
             new_comp = copy.deepcopy(comp)
-            has_active_ion = self.has_ion(from_ion, active_only=True)
             if logN:
                 new_comp.logN += offset_N
-            if tie_z and has_active_ion:
+            if tie_z:
                 new_comp.tie_z = 'z%i_%s' % (num, from_ion)
-            if tie_b and has_active_ion:
+            if tie_b:
                 new_comp.tie_b = 'b%i_%s' % (num, from_ion)
 
             self.components[to_ion].append(new_comp)
@@ -1291,29 +1145,28 @@ class DataSet(object):
     def load_components_from_file(self, fname, fit_pars=True):
         """Load best-fit parameters from an output file `fname`.
         If `fit_pars` is True, then update the best_fit parameters."""
+        parameters = open(fname)
         components_to_add = list()
         all_ions_in_file = list()
-        with open(fname) as parameters:
-            for line in parameters.readlines():
-                line = line.strip()
-                pars = line.split()
-                if len(line) == 0:
-                    pass
-                elif line[0] == '#':
-                    pass
-                elif len(pars) == 8:
-                    num = int(pars[0])
-                    ion = pars[1]
-                    z = float(pars[2])
-                    z_err = float(pars[3])
-                    b = float(pars[4])
-                    b_err = float(pars[5])
-                    logN = float(pars[6])
-                    logN_err = float(pars[7])
-                    components_to_add.append([num, ion, z, b, logN,
-                                              z_err, b_err, logN_err])
-                    if ion not in all_ions_in_file:
-                        all_ions_in_file.append(ion)
+        for line in parameters.readlines():
+            line = line.strip()
+            pars = line.split()
+            if len(line) == 0:
+                pass
+            elif line[0] == '#':
+                pass
+            elif len(pars) == 8:
+                ion = pars[1]
+                z = float(pars[2])
+                z_err = float(pars[3])
+                b = float(pars[4])
+                b_err = float(pars[5])
+                logN = float(pars[6])
+                logN_err = float(pars[7])
+                components_to_add.append([ion, z, b, logN,
+                                          z_err, b_err, logN_err])
+                if ion not in all_ions_in_file:
+                    all_ions_in_file.append(ion)
 
         for ion in all_ions_in_file:
             if ion in self.components.keys():
@@ -1327,8 +1180,9 @@ class DataSet(object):
                     for parname in pars_to_delete:
                         self.best_fit.pop(parname)
 
-        for comp_pars in components_to_add:
-            (num, ion, z, b, logN, z_err, b_err, logN_err) = comp_pars
+        for num, comp_pars in enumerate(components_to_add):
+            (ion, z, b, logN,
+             z_err, b_err, logN_err) = comp_pars
             self.add_component(ion, z, b, logN)
             if fit_pars and self.best_fit:
                 parlist = [['z', z, z_err],
@@ -1340,10 +1194,7 @@ class DataSet(object):
                     self.best_fit.add(parname, value=val)
                     self.best_fit[parname].stderr = err
 
-        if self.verbose:
-            print("\n  Added components for the given ions:")
-            print("  " + ", ".join(all_ions_in_file))
-
+        parameters.close()
 
     def fix_structure(self, ion=None):
         """Fix the velocity structure, that is, the redshifts and the b-parameters.
@@ -1670,9 +1521,6 @@ class DataSet(object):
 
     # =========================================================================
 
-    def add_variable(self, name, **kwargs):
-        self.static_variables.add(name, **kwargs)
-
     def prepare_dataset(self, norm=True, mask=False, verbose=True,
                         active_only=False,
                         force_clean=True,
@@ -1696,9 +1544,6 @@ class DataSet(object):
         verbose : bool   [default = True]
             If this is set, the code will print small info statements during the run.
 
-        active_only : bool   [default = False]
-            If True, only define masks for active lines.
-
         force_clean : bool   [default = False]
             If this is True, components for inactive elements will be removed.
 
@@ -1760,14 +1605,14 @@ class DataSet(object):
                 print("")
 
         # --- Check that no components for inactive elements are defined:
-        for this_ion in list(self.components.keys()):
-            lines_for_this_ion = [this_line.active for this_line in self.lines.values() if this_line.ion == this_ion]
+        for this_ion in self.components.keys():
+            lines_for_this_ion = [l.active for l in self.lines.values() if l.ion == this_ion]
 
             if np.any(lines_for_this_ion):
                 pass
             else:
-                if verbose:
-                    warn_msg = "\n [WARNING] - Components defined for inactive or missing element: %s"
+                if self.verbose:
+                    warn_msg = "\n [WARNING] - Components defined for inactive element: %s"
                     print(warn_msg % this_ion)
 
                 if force_clean:
@@ -1779,7 +1624,6 @@ class DataSet(object):
 
         # --- Prepare fit parameters  [class: lmfit.Parameters]
         self.pars = Parameters()
-        self.pars += self.static_variables
         # - First setup parameters with values only:
         for ion in self.components.keys():
             for n, comp in enumerate(self.components[ion]):
@@ -1790,7 +1634,8 @@ class DataSet(object):
                 N_name = 'logN%i_%s' % (n, ion)
 
                 self.pars.add(z_name, value=myfloat(z), vary=comp.var_z)
-                self.pars.add(b_name, value=myfloat(b), vary=comp.var_b, min=0.)
+                self.pars.add(b_name, value=myfloat(b), vary=comp.var_b,
+                              min=0.)
                 self.pars.add(N_name, value=myfloat(logN), vary=comp.var_N)
 
         # - Then setup parameter links:
@@ -1823,18 +1668,6 @@ class DataSet(object):
                     else:
                         self.pars.add('R%i_cheb_p%i' % (reg_num, cheb_num), value=0.0, vary=var_par)
 
-        # Check that all static variables are used:
-        all_constraints = "  ".join([p.expr for p in self.pars.values() if p.expr])
-        var_names = list(self.static_variables.keys())
-        for varname in var_names:
-            regex = r'(^|[^a-z^A-Z^0-9_.])[+,-,*,\/]?(%s)[+,-,*,\/]?([^a-z^A-Z^0-9_.]|$)' % varname
-            find_var = re.compile(regex)
-            if find_var.search(all_constraints) is None:
-                self.pars.pop(varname)
-                self.static_variables.pop(varname)
-                if self.verbose and verbose:
-                    print(" [INFO] - unused variable was removed: %s" % varname)
-
         # --- mask spectral regions that should not be fitted
         if mask:
             for region in self.regions:
@@ -1853,7 +1686,6 @@ class DataSet(object):
                 print("")
 
         self.ready2fit = True
-        plt.close('all')
 
         # --- Check that all active elements have components defined:
         for line_tag in self.all_lines:
@@ -1867,8 +1699,7 @@ class DataSet(object):
                 self.ready2fit = False
                 # TODO:
                 # automatically open interactive window if components are not defined.
-                error_msg = " [ERROR] - Components are not defined for element: %s \n" % ion
-                return error_msg
+                return False
 
         # -- Check all transitions of the given ions that are covered by the data:
         if check_lines:
@@ -1902,7 +1733,7 @@ class DataSet(object):
             if verbose and self.verbose:
                 print("\n  Dataset is ready to be fitted.")
                 print("")
-            return ""
+            return True
 
     def fit(self, verbose=True, **kwargs):
         """
@@ -1972,7 +1803,7 @@ class DataSet(object):
             print(" [WARNING] - rebinning for LSF file kernel is not supported!")
 
         if self.verbose:
-            print("\n  Fit is running... Please be patient.\n")
+            print("\n  Fit is running... Please, be patient.\n")
 
         def chi(pars):
             model = list()
@@ -1983,20 +1814,23 @@ class DataSet(object):
                 if region.has_active_lines():
                     x, y, err, mask = region.unpack()
                     if rebin > 1:
-                        x, y, err = output.rebin_spectrum(x, y, err, rebin)
-                        mask = output.rebin_bool_array(mask, rebin)
                         if isinstance(region.kernel, float):
+                            x, y, err = output.rebin_spectrum(x, y, err, rebin)
+                            mask = output.rebin_bool_array(mask, rebin)
                             nsub = region.kernel_nsub
                         else:
+                            x, y, err = output.rebin_spectrum(x, y, err, rebin)
+                            mask = output.rebin_bool_array(mask, rebin)
                             # Multiply the subsampling factor of the kernel by the rebin factor
                             nsub = region.kernel_nsub * rebin
                     else:
                         nsub = region.kernel_nsub
 
                     # Generate line profile
-                    profile_obs = evaluate_profile(x, pars, self.lines.values(),
-                                                   region.kernel, z_sys=self.redshift,
-                                                   sampling=sampling, kernel_nsub=nsub)
+                    profile_obs = evaluate_profile(x, pars, self.redshift,
+                                                   self.lines.values(), self.components,
+                                                   region.kernel, sampling=sampling,
+                                                   kernel_nsub=nsub)
 
                     if self.cheb_order >= 0:
                         cont_model = evaluate_continuum(x, pars, reg_num)
@@ -2042,15 +1876,15 @@ class DataSet(object):
 
             if verbose:
                 output.print_results(self, self.best_fit, velocity=True)
-                # if self.cheb_order >= 0:
-                #     output.print_cont_parameters(self)
+                if self.cheb_order >= 0:
+                    output.print_cont_parameters(self)
 
         chi2 = popt.chisqr
         return popt, chi2
 
     def plot_fit(self, rebin=1, fontsize=12, xmin=None, xmax=None, max_rows=4,
                  ymin=None, ymax=None, filename=None,
-                 subsample_profile=10, npad=50, loc='left',
+                 subsample_profile=1, npad=50, loc='left',
                  highlight_props=None, residuals=True, norm_resid=False,
                  default_props={}, element_props={}, legend=True,
                  label_all_ions=False, xunit='vel',
@@ -2292,128 +2126,3 @@ class DataSet(object):
                 vel = (z - z_sys) / (z_sys + 1) * 299792.458
                 print("   %2i  %+8.1f  %.6f   %6.1f   %5.2f" % (num, vel, z,
                                                                 comp.b, comp.logN))
-
-    def equivalent_width_limit(self, line_tag, ref=None, nofit=False, sigma=3., verbose=True, threshold=15):
-        """
-        Determine the equivalent width limit and corresponding limit on log(N).
-
-        Parameters
-        ----------
-        line_tag : str
-            The line ID of the absorption line for which the limit should be estimated.
-            (Must be a line in the line-list of VoigtFit. Ex.: FeII_1611, TiII_1910, etc.)
-
-        ref : str or `None`
-            The reference line used to determine the integration range in velocity space,
-            which contains 99.7% of the optical depth. By default, the strongest line of
-            the same ionization state (e.g., I, II, IV, etc.) in the dataset is used.
-            If the reference line has been fitted, the best-fit optical depth profile
-            will be used, otherwise the range is determined by the observed apparent
-            optical depth.
-
-        nofit : bool   [default = False]
-            If True, always use the observed apparent optical depth of the reference line
-            (`ref`, see above).
-
-        sigma : float   [default = 3]
-            The significance level of the inferred upper limit, by default 3 sigma (99.7%) is used.
-
-        verbose : bool   [default = True]
-            Print informative messages from the function?
-
-        threshold : float   [default = 15]
-            The continuum noise threshold used to infer the edge of the apparent optical depth
-            of the observed profile. The noise is estimated as the median noise per pixel
-
-        """
-        line = self.lines[line_tag]
-        verbose = self.verbose | verbose
-        regs_of_line = self.find_line(line_tag)
-        reg = regs_of_line[0]
-        if not reg.normalized:
-            reg.normalize(z_sys=self.redshift)
-
-        # Find a line that matches the ionization state
-        # and determine the velocity extent of the line
-        use_data = nofit
-        if (self.best_fit is not None) and not use_data:
-            if ref is not None:
-                line_match = self.lines[ref]
-            else:
-                line_match, msg = match_ion_state(line, self.lines.values())
-                if line_match is None:
-                    if verbose:
-                        print("           Could not find a matching line to determine velocity range.")
-                        print(" [ERROR] - Aborting the measurement of equivalent width!\n")
-                        return None
-            # Check if best-fit parameters exist for the given line:
-            if 'logN0_%s' % line_match.ion in self.best_fit:
-                reg_match_all = self.find_line(line_match.tag)
-                reg_match = reg_match_all[0]
-                if not reg_match.normalized:
-                    reg_match.normalize(z_sys=self.redshift)
-                wl_ref = np.linspace(reg_match.wl.min(), reg_match.wl.max(), len(reg_match.wl)*10)
-                lcen = line_match.l0 * (self.redshift + 1)
-                vel_ref = (wl_ref - lcen) / lcen * 299792.458
-                # vel_ref = reg_match.get_velocity(self.redshift, line_match.tag)
-                profile = reg_match.evaluate_region(self.best_fit, wl=wl_ref, lines=[line_match])
-                tau = -np.log(profile)
-                vmin, vmax = tau_percentile(vel_ref, tau)
-                use_data = False
-                if verbose:
-                    print("\n [INFO] - Determining limit for %s, using the fitted profile of %s as reference" % (line_tag, line_match.tag))
-                    print(" [INFO] - Integrating from vel = %.1f to %.1f km/s\n" % (vmin, vmax))
-            else:
-                use_data = True
-
-        if use_data:
-            if ref is not None:
-                line_match = self.lines[ref]
-                reg_match_all = self.find_line(line_match.tag)
-                reg_match = reg_match_all[0]
-            else:
-                matches = match_ion_state_all(line, self.lines.values())
-                line_strength = [ll.l0 * ll.f for ll in matches]
-                # Loop through the lines sorted by line-strength (strongest to weakest):
-                for _, this_line in sorted(zip(line_strength, matches), reverse=True):
-                    these_regs = self.find_line(this_line.tag)
-                    this_reg = these_regs[0]
-                    if len(this_reg.lines) > 1:
-                        continue
-                    flux = this_reg.flux
-                    mask = this_reg.mask
-                    if np.min(flux[mask]) > 0:
-                        line_match = this_line
-                        reg_match = this_reg
-                        break
-                else:
-                    if verbose:
-                        print("           Could not find a matching line to determine velocity range.")
-                        print(" [ERROR] - Aborting the measurement of equivalent width!\n")
-                    return None
-            if not reg_match.normalized:
-                reg_match.normalize(z_sys=self.redshift)
-            vel_ref = reg_match.get_velocity(self.redshift, line_match.tag)
-            _, flux, err, mask = reg_match.unpack()
-            tau = -np.log(flux[mask])
-            tau_err = err[mask] / flux[mask]
-            noise = np.median(tau_err) * threshold
-            vmin, vmax = tau_noise_range(vel_ref[mask], tau, noise)
-            if verbose:
-                print("\n [INFO] - Determining limit for %s, using the observed profile of %s as reference" % (line_tag, line_match.tag))
-                print(" [INFO] - Integrating from vel = %.1f to %.1f km/s\n" % (vmin, vmax))
-
-        vel = reg.get_velocity(self.redshift, line_tag)
-        aper = (vel > vmin) & (vel < vmax)
-        W_rest, W_err = equivalent_width(reg.wl, reg.flux, reg.err, aper=aper, z_sys=self.redshift)
-        W_limit = sigma * W_err
-        logN_limit = np.log10(1.13e20*W_limit / (line.l0**2 * line.f))
-        logN = np.log10(1.13e20*W_rest / (line.l0**2 * line.f))
-        logN_err = W_err / (W_rest * np.log(10))
-
-        result = EquivalentWidth(W_rest=W_rest, W_err=W_err, logN=logN, logN_err=logN_err,
-                                 logN_limit=logN_limit, line=line_tag, sigma=sigma)
-
-        limit_fname = self.name + '_%s_limit.pdf' % line_tag
-        output.plot_limit(self, line, line_match, vel_ref, tau, vmin, vmax, use_data, filename=limit_fname, EW=result)
-        return result
